import pygame
from pygame.locals import *
import core
 
class cFontManager:
    '''
    A simple class used to manage Font objects and provide a simple way to use
    them to draw text on any surface.
 
    Directly import this file to use the class, or run this file from the
    command line to see a trivial sample.
 
    Written by Scott O. Nelson  
    '''
    def __init__(self, listOfFontNamesAndSizesAsTuple):
        '''
        Pass in a tuple of 2-item tuples.  Each 2-item tuple is a fontname / 
        size pair. To use the default font, pass in a None for the font name.
        Font objects are created for each of the pairs and can then be used
        to draw text with the Draw() method below.
        
        Ex: fontMgr = cFontManager(((None, 24), ('arial', 18), ('arial', 24),
            ('courier', 12), ('papyrus', 50)))
 
        TODO: add support for bold & italics
        '''
        self._fontDict = {}
        for pair in listOfFontNamesAndSizesAsTuple:
            assert len(pair) == 2, \
                "Pair must be composed of a font name and a size - ('arial', 24)"
            if pair[0]:
                fontFullFileName = pygame.font.match_font(pair[0])
                assert fontFullFileName, 'Font: %s Size: %d is not available.' % pair
            else:
                fontFullFileName = None # use default font
            self._fontDict[pair] = pygame.font.Font(fontFullFileName, pair[1])
 
    def Draw(self, surface, fontName, size, text, rectOrPosToDrawTo, color,
            alignHoriz='left', alignVert='top', antialias=False):
        '''
        Draw text with the given parameters on the given surface. Return
        a rect.
        
        surface - Surface to draw the text onto.
        
        fontName - Font name that identifies what font to use to draw the text.
        This font name must have been specified in the cFontManager 
        
        rectOrPosToDrawTo - Where to render the text at.  This can be a 2
        item tuple or a Rect.  If a position tuple is used, the align
        arguments will be ignored.
        
        color - Color to draw the text with.
        
        alignHoriz - Specifies horizontal alignment of the text in the
        rectOrPosToDrawTo Rect.  If rectOrPosToDrawTo is not a Rect, the
        alignment is ignored.
        
        alignVert - Specifies vertical alignment of the text in the
        rectOrPosToDrawTo Rect.  If rectOrPosToDrawTo is not a Rect, the
        alignment is ignored.
 
        antialias - Whether to draw the text anti-aliased or not.
        '''
        pair = (fontName, size)
        assert self._fontDict.has_key(pair), \
            'Font: %s Size: %d is not available in cFontManager.' % pair
        fontSurface = self._fontDict[(fontName, size)].render(text,
            antialias, color)
        if isinstance(rectOrPosToDrawTo, tuple):
            return surface.blit(fontSurface, rectOrPosToDrawTo)
        elif isinstance(rectOrPosToDrawTo, pygame.Rect):
            fontRect = fontSurface.get_rect()
            # align horiz
            if alignHoriz == 'center':
                fontRect.centerx = rectOrPosToDrawTo.centerx
            elif alignHoriz == 'right':
                fontRect.right = rectOrPosToDrawTo.right
            else:
                fontRect.x = rectOrPosToDrawTo.x  # left
            # align vert
            if alignVert == 'center':
                fontRect.centery = rectOrPosToDrawTo.centery
            elif alignVert == 'bottom':
                fontRect.bottom = rectOrPosToDrawTo.bottom
            else:
                fontRect.y = rectOrPosToDrawTo.y  # top
                
            return surface.blit(fontSurface, fontRect)
 
# A really simple and dummy SFont manager (because that's all I want nao)
class oSFont:
    def __init__(self, image, charset):
        self.charset = {}
        self.spacing = 2
        image, margins = self.getMargins(image)
        self.initialize(image, charset, margins)
        
    def initialize(self, image, charset, margins):
        self.width = image.get_width() / len(charset)
        self.height = image.get_height()
        for char in charset:
            offx, width = margins.pop(0)
            self.charset[char] = core.im_crop(image, pygame.Rect(offx, 0, width, self.height))
                
         
    def getWidth(self, text):
        rv = 0
        for char in text:
            if self.charset.has_key(char):
                rv += self.charset[char].get_width() + self.spacing
        return rv
            
    def draw(self, surface, pos, text):
        x, y = pos
        for char in text:
            if self.charset.has_key(char):
                surface.blit( self.charset[char], (x,y) )
                x += self.charset[char].get_width() + self.spacing
                
        return pygame.Rect(pos, (x, y+self.height))


    def getMargins(self, image):
        margins   = []
        w, h = image.get_size()
        transp = (255,0,255,255)
        xo = -1
        cw = 0
        for x in range(w):
            if image.get_at((x, h-1)) == transp:
                image.set_at((x, h-1), (0,0,0,0))
                if xo > -1:
                    cw = x - xo
                    margins.append([xo, cw])
                    xo = -1
            else:
                if xo == -1:
                    xo = x
        
        return image, margins
    
    